pub trait MapKey: Sealed { }Expand description
A type that map keys can be looked up with.
This is implemented for strings, integers, bools, chars and
Values. Looking up keys with it does not require the key to be
converted into a value first.
Dyn Compatibility
This trait is dyn compatible.
In older versions of Rust, dyn compatibility was called "object safety".