pub struct Map { /* private fields */ }Expand description
A map of values.
Maps retain the order in which the entries were inserted and every key
is unique. Keys can be any value, not just strings. Additionally a
map holds the Order of its entries and if it’s a multimap (see
is_multimap) which are passed on to formats when
the map is serialized. Neither is considered when maps are compared:
maps are equal if they contain the same entries. The
exception are entries with maps or sequences as keys, which are compared
in order (this allows comparing maps without recursion).
use deser_value::{Map, Value};
let mut map = Map::new();
map.insert("name", "Jane");
map.insert(42, true);
assert_eq!(map.get("name"), Some(&Value::from("Jane")));
assert_eq!(map.get(&42), Some(&Value::from(true)));
assert_eq!(
map.keys().collect::<Vec<_>>(),
[&Value::from("name"), &Value::from(42)]
);Keys can be looked up by anything that implements MapKey, which
includes strings, integers and values.
Implementations
Sourceimpl Map
impl Map
Sourcepub fn with_capacity(capacity: usize) -> Map
pub fn with_capacity(capacity: usize) -> Map
Creates an empty map with a capacity.
Sourcepub fn is_multimap(&self) -> bool
pub fn is_multimap(&self) -> bool
Returns true if the map is a multimap.
Maps that are deserialized from multimaps (like query strings, see
ContainerShape::set_multimap)
are multimaps. The values of a key that was given more than once
are a Seq marked as
repeated. When the map is deserialized
into another type, it’s a multimap again and the values of repeated
keys are passed on as the values of repeated keys. A key that was
given once is a single value, so types that collect the values of
keys (like Vec<T>) receive the same values as from the original
input.
use deser_value::{Map, Seq, Value};
#[derive(deser::Deserialize, Debug, PartialEq)]
struct Query {
tag: Vec<String>,
page: Vec<u32>,
}
let mut map = Map::new();
map.set_multimap(true);
let mut tags = Seq::from(vec![Value::from("a"), Value::from("b")]);
tags.set_repeated(true);
map.insert("tag", tags);
map.insert("page", 1);
let query: Query = deser_value::from_value(&Value::from(map)).unwrap();
assert_eq!(
query,
Query { tag: vec!["a".into(), "b".into()], page: vec![1] }
);Sourcepub fn set_multimap(&mut self, yes: bool)
pub fn set_multimap(&mut self, yes: bool)
Sets if the map is a multimap.
Sourcepub fn is_ambiguous_empty(&self) -> bool
pub fn is_ambiguous_empty(&self) -> bool
Returns true if the map is empty and could also be an empty
sequence.
This is the counterpart of
Seq::is_ambiguous_empty: types
that expect a sequence receive an empty sequence. The flag has no
effect once the map has entries.
Sourcepub fn set_ambiguous_empty(&mut self, yes: bool)
pub fn set_ambiguous_empty(&mut self, yes: bool)
Sets if the map could also be an empty sequence when it’s empty.
Sourcepub fn get_mut<K: MapKey + ?Sized>(&mut self, key: &K) -> Option<&mut Value>
pub fn get_mut<K: MapKey + ?Sized>(&mut self, key: &K) -> Option<&mut Value>
Returns the value of a key mutably.
Sourcepub fn get_key_value<K: MapKey + ?Sized>(
&self,
key: &K,
) -> Option<(&Value, &Value)>
pub fn get_key_value<K: MapKey + ?Sized>( &self, key: &K, ) -> Option<(&Value, &Value)>
Returns the key and value of a key.
This is useful to get to the meta data of the key.
Sourcepub fn get_index_of<K: MapKey + ?Sized>(&self, key: &K) -> Option<usize>
pub fn get_index_of<K: MapKey + ?Sized>(&self, key: &K) -> Option<usize>
Returns the index of a key.
Sourcepub fn get_index(&self, index: usize) -> Option<(&Value, &Value)>
pub fn get_index(&self, index: usize) -> Option<(&Value, &Value)>
Returns the key and value at an index.
Sourcepub fn get_index_mut(&mut self, index: usize) -> Option<(&Value, &mut Value)>
pub fn get_index_mut(&mut self, index: usize) -> Option<(&Value, &mut Value)>
Returns the key and the mutable value at an index.
Sourcepub fn contains_key<K: MapKey + ?Sized>(&self, key: &K) -> bool
pub fn contains_key<K: MapKey + ?Sized>(&self, key: &K) -> bool
Returns true if the map contains a key.
Sourcepub fn insert<K: Into<Value>, V: Into<Value>>(
&mut self,
key: K,
value: V,
) -> Option<Value>
pub fn insert<K: Into<Value>, V: Into<Value>>( &mut self, key: K, value: V, ) -> Option<Value>
Inserts a value for a key.
If the key already exists, its value is replaced (the entry keeps its position and key) and the old value is returned. Otherwise the entry is added at the end.
Sourcepub fn get_or_insert_with<K: Into<Value>, F: FnOnce() -> Value>(
&mut self,
key: K,
f: F,
) -> &mut Value
pub fn get_or_insert_with<K: Into<Value>, F: FnOnce() -> Value>( &mut self, key: K, f: F, ) -> &mut Value
Returns the value of a key, inserting the result of f if the key
does not exist.
Sourcepub fn remove<K: MapKey + ?Sized>(&mut self, key: &K) -> Option<Value>
pub fn remove<K: MapKey + ?Sized>(&mut self, key: &K) -> Option<Value>
Removes a key and returns its value.
The order of the remaining entries is retained.
Sourcepub fn remove_entry<K: MapKey + ?Sized>(
&mut self,
key: &K,
) -> Option<(Value, Value)>
pub fn remove_entry<K: MapKey + ?Sized>( &mut self, key: &K, ) -> Option<(Value, Value)>
Removes a key and returns it together with its value.
The order of the remaining entries is retained.
Sourcepub fn retain<F: FnMut(&Value, &mut Value) -> bool>(&mut self, f: F)
pub fn retain<F: FnMut(&Value, &mut Value) -> bool>(&mut self, f: F)
Retains the entries for which the predicate returns true.
Sourcepub fn iter_mut(&mut self) -> IterMut<'_> ⓘ
pub fn iter_mut(&mut self) -> IterMut<'_> ⓘ
Returns an iterator over the entries with mutable values.
Sourcepub fn values_mut(&mut self) -> ValuesMut<'_> ⓘ
pub fn values_mut(&mut self) -> ValuesMut<'_> ⓘ
Returns an iterator over the mutable values.